/*********************************************************************
 *
 * file name : discoverd.c
 * summary   : program to reply received armadillo-J discover packet
 * coded by  : mori
 * copyright : Atmark Techno
 * date(init): 2003.12.04
 * 
 ********************************************************************/

#include <iostream>
#include <cstdio>
#include <string.h>
#include <cstdlib>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <sys/uio.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <unistd.h>
#include <netinet/in.h>
#include <netdb.h>
#include <net/route.h>
#include <net/if.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <popt.h>

#include "discover_lib.h"

/* MACRO define */
#define TO_INADDR(x)	(*(struct in_addr*)&x)
#define OUTPUT_FORMAT	"%02x:%02x:%02x:%02x:%02x:%02x %s %s %s %s"
#define PRETTY_FORMAT	"%02x:%02x:%02x:%02x:%02x:%02x|DHCP %-3s|IP %-15s|MASK %-15s|GW %-15s"

using namespace std;

static int verbose = 0;
static int pretty = 0;
static int help = 0;
static int usage = 0;
static int timeout = 2000;
static int retry = 3;
static const struct poptOption options[] = {
	{"timeout", '\0', POPT_ARG_INT, &timeout, 0, "Timeout in msec (default: 2sec)", NULL},
	{"retry", '\0', POPT_ARG_INT, &retry, 0, "Retry sending discover packet (default: 3)", NULL},
        {"verbose", '\0', POPT_ARG_NONE, &verbose, 0, "Verbose output", NULL},
        {"pretty", '\0', POPT_ARG_NONE, &pretty, 0, "Pretty Print", NULL},
        POPT_AUTOHELP
        {NULL, '\0', 0, NULL, 0},
};

/**********************************************************************
 * summary : Output network info to find
 * return  :
 *********************************************************************/
void PrintNetworkInfo(const discover_packet_s* packet)
{
	char ip_address[16];
	char netmask[16];
	char gateway[16];

	strcpy(ip_address, inet_ntoa(TO_INADDR(packet->net_info.ip_address)));
	strcpy(netmask, inet_ntoa(TO_INADDR(packet->net_info.netmask)));
	strcpy(gateway, inet_ntoa(TO_INADDR(packet->net_info.gateway)));

        printf(pretty ? PRETTY_FORMAT : OUTPUT_FORMAT,
               (unsigned char)packet->mac_addr[0],
               (unsigned char)packet->mac_addr[1],
               (unsigned char)packet->mac_addr[2],
               (unsigned char)packet->mac_addr[3],
               (unsigned char)packet->mac_addr[4],
               (unsigned char)packet->mac_addr[5],
               packet->net_info.use_dhcp? "yes" : "no",
               ip_address,
               netmask,
               gateway);
        printf("\n");
}

/**********************************************************************
 * summary : main routine
 * return  : success -> 0, failed -> !0
 *********************************************************************/
int main(int argc, char *argv[])
{
        int rc;
	int i, j;
        poptContext poptcon;
	CDiscover discover;
	const char *iface;
	struct ifreq *ifr;
	int nr_ifs;

        poptcon = poptGetContext(NULL, argc, (const char **)argv, options, 0);
	poptSetOtherOptionHelp(poptcon, "[OPTION...] [interface]");
        while ((rc = poptGetNextOpt(poptcon)) != -1) {
                switch(rc) {
                case POPT_ERROR_NOARG:
                case POPT_ERROR_BADOPT:
                case POPT_ERROR_BADNUMBER:
                case POPT_ERROR_OVERFLOW:
                        printf("%s: bad argument %s: %s\n", argv[0],
                               poptBadOption(poptcon, 0), poptStrerror(rc));
                        poptFreeContext(poptcon);
                        exit(1);
                        break;
                }
        }
	iface = poptGetArg(poptcon);

        poptFreeContext(poptcon);

        if (help) {
                poptPrintHelp(poptcon, stdout, 0);
                return 0;
        }
        if (usage) {
                poptPrintHelp(poptcon, stdout, 0);
                return 0;
        }

	nr_ifs = discover.GetInterfaceList(&ifr);
	if(nr_ifs < 0){
		cerr << "failed to get network interface" << endl;
		return false;
	}

	for(i = 0; i < nr_ifs; i++){
		if(!strcmp(ifr[i].ifr_name, "lo"))
			continue;
		if((iface != NULL) && strcmp(ifr[i].ifr_name, iface))
			continue;

		for(j = 0; j < retry; j++){
			LOG("sending find packet\n");

			uint32_t transactionID  = discover.SendFindPacket(ifr[i].ifr_name);
			if(!transactionID){
				cerr << "failed to send fine packet" << endl;
				return -1;
			}

			while(1){
				discover_packet_s packet;
				LOG("wait for reply\n");
				if(discover.RecvReplyPacket(&packet, transactionID, timeout)){
					if (verbose)
						printf("packet received\n");
					if(packet.message_type == ntohl(DISCOVER_REP)){
						PrintNetworkInfo(&packet);
					}
				}
				else
					break;
			}
		}
	}

	discover.FreeInterfaceList(ifr);

	return 0;
}

